from decimal import Decimal

from django.db import models
from django.db.models import Sum
from django.db.models.functions import Coalesce

from minigest.pos.models import Cassa

from ...helpers.choices import ANALOG, FORMATO_CHIUSURA, XML6, XML7


class ChiusuraFiscale(models.Model):
    cassa = models.ForeignKey(Cassa, on_delete=models.CASCADE, related_name="chiusure")
    formato = models.CharField(max_length=10, choices=FORMATO_CHIUSURA, default=XML7)
    data = models.DateField()
    numero_documenti_commerciali = models.IntegerField(default=0)
    ingressi = models.IntegerField(default=0)

    @property
    def totale(self):
        reparti = self.reparti_iva.all().aggregate(
            totale=Coalesce(Sum("corrispettivo"), Decimal(0))
        )
        if self.formato == XML7:
            return Decimal(reparti["totale"]) + self.totale_annulli + self.totale_resi
        else:
            return Decimal(reparti["totale"])

    @property
    def totale_annulli(self):
        annulli = self.annulli.all().aggregate(
            totale=Coalesce(Sum("corrispettivo"), Decimal(0))
        )
        return annulli["totale"]

    @property
    def totale_resi(self):
        resi = self.resi.all().aggregate(
            totale=Coalesce(Sum("corrispettivo"), Decimal(0))
        )
        return resi["totale"]

    @property
    def imposta(self):
        totale = 0
        for reparto in self.reparti_iva.all():
            totale += round(reparto.imposta, 2)

        for reso in self.resi.all():
            if self.formato != XML7:
                totale -= round(reso.imposta, 2)

        for annullo in self.annulli.all():
            if self.formato != XML7:
                totale -= round(annullo.imposta, 2)

        return Decimal(totale)

    @property
    def imponibile(self):
        totale = 0
        for reparto in self.reparti_iva.all():
            totale += round(reparto.imponibile, 2)

        for reso in self.resi.all():
            if self.formato != XML7:
                totale -= round(reso.imponibile, 2)

        for annullo in self.annulli.all():
            if self.formato != XML7:
                totale -= round(annullo.imponibile, 2)

        return Decimal(totale)

    @property
    def gran_totale(self):
        reparti = self.reparti_iva.all().aggregate(
            totale=Coalesce(Sum("corrispettivo"), Decimal(0))
        )
        annulli = self.annulli.all().aggregate(
            totale=Coalesce(Sum("corrispettivo"), Decimal(0))
        )
        resi = self.resi.all().aggregate(
            totale=Coalesce(Sum("corrispettivo"), Decimal(0))
        )

        if self.formato == XML7:
            return reparti["totale"]
        else:
            return reparti["totale"] - annulli["totale"] - resi["totale"]

    def __str__(self):
        data = self.data
        return f"Chiusura Fiscale del {data} - {self.gran_totale}€"

    def save(self, *args, **kwargs):
        if self.ingressi == 0 and self.numero_documenti_commerciali > 0:
            self.ingressi = self.numero_documenti_commerciali
        super(ChiusuraFiscale, self).save(*args, **kwargs)

    class Meta:
        verbose_name_plural = "Chiusure Fiscali"
        ordering = ["-data"]
